@@ -71,20 +71,37 @@ expression ::= <start> '{' <expression_inner>:e '}' => ('expand', ) + e
 escapedexpression ::= <start> <expression_inner>:e => ('escapedexpand', ) + e
 block_inner ::= <spaces> <symbol>:s <arguments>:args <spaces> <finish>
     => (u''.join(s), args)
+partial_inner ::= <spaces> <partialname>:s <arguments>:args <spaces> <finish>
+    => (u''.join(s), args)
 alt_inner ::= <spaces> ('^' | 'e' 'l' 's' 'e') <spaces> <finish>
-partial ::= <start> '>' <block_inner>:i => ('partial',) + i
+partial ::= <start> '>' <partial_inner>:i => ('partial',) + i
 path ::= ~('/') <pathseg>+:segments => ('path', segments)
-kwliteral ::= <symbol>:s '=' (<literal>|<path>):v => ('kwparam', s, v)
+kwliteral ::= <safesymbol>:s '=' (<literal>|<path>):v => ('kwparam', s, v)
 literal ::= (<string>|<integer>|<boolean>):thing => ('literalparam', thing)
-string ::= '"' <notquote>*:ls '"' => u'"' + u''.join(ls) + u'"'
+string ::= '"' <notdquote>*:ls '"' => u'"' + u''.join(ls) + u'"'
+    | "'" <notsquote>*:ls "'" => u"'" + u''.join(ls) + u"'"
 integer ::= '-'?:sign <digit>+:ds => int((sign if sign else '') + ''.join(ds))
 boolean ::= <false>|<true>
 false ::= 'f' 'a' 'l' 's' 'e' => False
 true ::= 't' 'r' 'u' 'e' => True
-notquote ::= <escapedquote> | (~('"') <anything>)
+notdquote ::= <escapedquote>
+    | '\n' => '\\n'
+    | '\r' => '\\r'
+    | '\\' => '\\\\'
+    | (~('"') <anything>)
+notsquote ::= <escapedquote>
+    | '\n' => '\\n'
+    | '\r' => '\\r'
+    | '\\' => '\\\\'
+    | (~("'") <anything>)
 escapedquote ::= '\\' '"' => '\\"'
+    | "\\" "'" => "\\'"
+notclosebracket ::= (~(']') <anything>)
+safesymbol ::=  ~<alt_inner> '['? (<letter>|'_'):start (<letterOrDigit>|'_')+:symbol ']'? => start + u''.join(symbol)
 symbol ::=  ~<alt_inner> '['? (<letterOrDigit>|'-'|'@')+:symbol ']'? => u''.join(symbol)
-pathseg ::= ('@' '.' '.' '/') => u'@@_parent'
+partialname ::= ~<alt_inner> ('['|'"')? (~(<space>|<finish>|']'|'"' ) <anything>)+:symbol (']'|'"')? => u''.join(symbol)
+pathseg ::= '[' <notclosebracket>+:symbol ']' => u''.join(symbol)
+    | ('@' '.' '.' '/') => u'@@_parent'
     | <symbol>
     | '/' => u''
     | ('.' '.' '/') => u'@_parent'
@@ -320,6 +337,13 @@ def _unless(this, options, context):
         return options['fn'](this)
 
 
+def _lookup(this, context, key):
+    try:
+        return context[key]
+    except (KeyError, IndexError, TypeError):
+        return
+
+
 def _blockHelperMissing(this, options, context):
     # print this, context
     if isinstance(context, collections.Callable):
@@ -355,6 +379,7 @@ _pybars_ = {
         'log': _log,
         'unless': _unless,
         'with': _with,
+        'lookup': _lookup,
     },
 }
 
