@@ -509,6 +509,13 @@ class TestAcceptance(TestCase):
         self.assertEqual("Dudes: Jeepers",
             render(source, context, partials=dict(dude=dude_src)))
 
+    def test_partials_with_string(self):
+        source = u'Dudes: {{> "+404/asdf?.bar"}}';
+        dude_src = u"{{name}}"
+        context = {'name': "Jeepers", 'another_dude': "Creepers"}
+        self.assertEqual("Dudes: Jeepers",
+            render(source, context, partials={'+404/asdf?.bar':dude_src}))
+
     def test_simple_literals_work(self):
         source = u'Message: {{hello "world" 12 true false}}'
         def hello(this, param, times, bool1, bool2):
@@ -535,7 +542,7 @@ class TestAcceptance(TestCase):
             render(source, {}, helpers=dict(hello=hello)))
 
     def test_it_works_with_single_quote_marks(self):
-        source = u'Message: {{{hello "Alan\'s world"}}}'
+        source = u"Message: {{{hello 'Alan\\\'s world'}}}"
         def hello(this, param):
             return "Hello " + param
         self.assertEqual("Message: Hello Alan's world",
@@ -891,6 +898,42 @@ class TestAcceptance(TestCase):
             render(u'{{#goodbye cruel="CRUEL" _print=false}}world{{/goodbye}}',
                 {}, helpers=helpers))
 
+    def test_should_lookup_arbitrary_content(self):
+        string  = u'{{#each goodbyes}}{{lookup ../data .}}{{/each}}'
+        context = {'goodbyes': [0, 1], 'data': ['foo', 'bar']}
+
+        template = Compiler().compile(string)
+        result   = template(context)
+
+        self.assertEqual(str_class(result), 'foobar')
+
+    def test_should_not_fail_on_undefined_value(self):
+        string  = u'{{#each goodbyes}}{{lookup ../bar .}}{{/each}}'
+        context = {'goodbyes': [0, 1], 'data': ['foo', 'bar']}
+
+        template = Compiler().compile(string)
+        result = template(context)
+
+        self.assertEqual(str_class(result), '')
+
+    def test_should_not_fail_on_unavailable_value(self):
+        string  = u'{{lookup thelist 3}}.{{lookup theobject "qux"}}.{{lookup thenumber 0}}'
+        context = {'thelist': ['foo', 'bar'], 'theobject': {'foo': 'bar'}, 'thenumber': 7}
+
+        template = Compiler().compile(string)
+        result = template(context)
+
+        self.assertEqual(str_class(result), '..')
+
+    def test_should_lookup_content_by_special_variables(self):
+        string  = u'{{#each goodbyes}}{{lookup ../data @index}}{{/each}}'
+        context = {'goodbyes': [0, 1], 'data': ['foo', 'bar']}
+
+        template = Compiler().compile(string)
+        result   = template(context)
+
+        self.assertEqual(str_class(result), 'foobar')
+
     def test_GH_94_Cannot_read_property_of_undefined(self):
         context = {"books": [
             {"title": "The origin of species",
@@ -941,3 +984,35 @@ class TestAcceptance(TestCase):
             }
         expected = "<strong>This is a slightly more complicated blah.</strong>.\n\nCheck this out:\n\n<ul>\n\n<li class=one>@fat</li>\n\n<li class=two>@dhg</li>\n\n<li class=three>@sayrer</li>\n</ul>.\n\n"
         self.assertEqual(expected, render(source, context))
+
+    def test_invalid_python_identifiers_cannot_be_used_as_keyword_arguments(self):
+        source = u'{{foo 0x="bar"}}'
+        self.assertEqual('', render(source, {}))
+
+    def test_backslash_does_not_normally_escape_text(self):
+        source = u'{{echo "\\x"}}'
+        self.assertEqual('\\x', render(source, {}, helpers={'echo': (lambda this, arg: arg)}))
+
+    def test_backslash_only_escapes_quote(self):
+        # Notice that the following is a raw string. It is a helper, 'echo',
+        # with a string argument equal to a backslash and an escaped quote.
+        source = u'{{echo "\\\\""}}'
+        # If the parser does not know to escape the backslash but does know to
+        # escape the quote, it will end up with something like the following
+        # in our generated rendering code:
+        #
+        #     value = value(child_scope, "\\"")
+        #
+        # Which will raise a SyntaxError.
+        self.assertEqual('\\&quot;', render(source, {}, helpers={'echo': (lambda this, arg: arg)}))
+
+    def test_newlines_in_string_litereals(self):
+        source = u'{{echo "Hello,\nWorld!"}}'
+        self.assertEqual('Hello,\nWorld!', render(source, {}, helpers={'echo': (lambda this, arg: arg)}))
+
+    def test_code_injection(self):
+        # If esape sequences are not dealt with properly, we are able to run
+        # arbitrary Python code. Thanks to @thomasst for pointing this out:
+        # https://github.com/elasticsales/pybars/commit/3a262b8cd7902889cde5c786f76bb0c30f8894e6#commitcomment-4687173
+        source = u'{{echo "\\\\")\n\n        raise AssertionError(\'Code Injected!\')\n#"}}'
+        self.assertEqual('\\&quot;)\n\n        raise AssertionError(&#x27;Code Injected!&#x27;)\n#', render(source, {}, helpers={'echo': (lambda this, arg: arg)}))
